=== modified file 'Makefile'
--- Makefile	2012-01-10 11:16:56 +0000
+++ Makefile	2012-01-11 08:06:03 +0000
@@ -1,2 +1,4 @@
 check:
 	python -m unittest grackle.tests.test_client
+
+.PHONY: check

=== modified file 'grackle/client.py'
--- grackle/client.py	2012-01-10 14:41:33 +0000
+++ grackle/client.py	2012-01-11 13:37:23 +0000
@@ -1,10 +1,11 @@
 import httplib
+import simplejson
 from urlparse import urlunparse
-from urllib import quote
-
-
-def message_path(archive_name, message_id):
-    return '/%s/%s' % (quote(archive_name), quote(message_id))
+from urllib import (
+    quote,
+    urlencode,
+)
+
 
 
 class GrackleClient:
@@ -12,17 +13,25 @@
     def __init__(self, host, port):
         self.host = host
         self.port = port
-
-    def message_url(self, archive_name, message_id):
-        netloc = '%s:%d' % (self.host, self.port)
-        path = message_path(archive_name, message_id)
-        return urlunparse(('http', netloc, path, '', '', ''))
-
-    def put_message(self, archive_name, message_id, file_obj):
-        url = self.message_url(archive_name, message_id)
-        connection = httplib.HTTPConnection(self.host, self.port)
-        connection.request('PUT', url, file_obj.read())
-        response = connection.getresponse()
+        self.netloc = '%s:%d' % (host, port)
+
+    def archive_url(self, archive_name, query):
+        path = '/%s' % quote(archive_name)
+        query = urlencode(query)
+        return urlunparse(('http', self.netloc, path, '', query, ''))
+
+    def _get_connection(self):
+        return httplib.HTTPConnection(self.host, self.port)
+
+    def _verb_archive(self, verb, archive_name, query, body=None):
+        url = self.archive_url(archive_name, query)
+        connection = self._get_connection()
+        connection.request(verb, url, body)
+        return connection.getresponse()
+
+    def put_message(self, archive_name, key, file_obj):
+        response = self._verb_archive(
+            'POST', archive_name, {'key': key}, file_obj.read())
         data = response.read()
         if response.status == httplib.BAD_REQUEST:
             raise Exception('wtf')
@@ -30,3 +39,12 @@
             return
         else:
             raise Exception('!!')
+
+    def get_messages(self, archive_name, message_ids=None):
+        parameters = {}
+        if message_ids is not None:
+            parameters['message_ids'] = message_ids
+        query = {'parameters': simplejson.dumps(parameters)}
+        response = self._verb_archive('GET', archive_name, query)
+        data = response.read()
+        return simplejson.loads(data)

=== modified file 'grackle/tests/test_client.py'
--- grackle/tests/test_client.py	2012-01-10 14:44:37 +0000
+++ grackle/tests/test_client.py	2012-01-11 13:37:23 +0000
@@ -5,31 +5,42 @@
 import httplib
 import os
 from signal import SIGKILL
+import simplejson
 from StringIO import StringIO
 from unittest import TestCase
+from urlparse import urlparse
 
 from testtools import ExpectedException
 
 from grackle.client import (
     GrackleClient,
-    message_path,
     )
 
 
-class Forked:
+class ForkedFake:
 
-    def __init__(self, func_or_method, *args):
-        self.func_or_method = func_or_method
+    def __init__(self, port, messages=None):
         self.pid = None
-        self.args = args
+        self.port = port
+        self.messages = messages
+        self.read_end, self.write_end = os.pipe()
+
+    def is_ready(self):
+        os.write(self.write_end, 'asdf')
 
     def __enter__(self):
         pid = os.fork()
-        if pid != 0:
-            self.pid = pid
-            return
-        self.func_or_method(*self.args)
+        if pid == 0:
+            self.start_server()
+        self.pid = pid
+        os.read(self.read_end, 1)
+        return
 
+    def start_server(self):
+        service = HTTPServer(('', self.port), FakeGrackleRequestHandler)
+        service.messages = self.messages
+        self.is_ready()
+        service.serve_forever()
 
     def __exit__(self, exc_type, exc_val, traceback):
         os.kill(self.pid, SIGKILL)
@@ -37,7 +48,7 @@
 
 class FakeGrackleRequestHandler(BaseHTTPRequestHandler):
 
-    def do_PUT(self):
+    def do_POST(self):
         message = self.rfile.read(int(self.headers['content-length']))
         if message == 'This is a message':
             self.send_response(httplib.CREATED)
@@ -46,25 +57,39 @@
         else:
             self.send_error(httplib.BAD_REQUEST)
 
-
-def run_service(port):
-    service = HTTPServer(('', port), FakeGrackleRequestHandler)
-    service.serve_forever()
-
+    def do_GET(self):
+        scheme, netloc, path, params, query, fragments = urlparse(self.path)
+        archive = os.path.split(path)[1]
+        self.send_response(httplib.OK)
+        self.end_headers()
+        self.wfile.write(simplejson.dumps(self.server.messages[archive]))
+
+
+def fake_grackle_service(client, messages=None):
+    if messages is None:
+        messages = {}
+    return ForkedFake(client.port, messages)
 
 
 class TestPutMessage(TestCase):
 
     def test_put_message(self):
-        client = GrackleClient('localhost', 8435)
-        with Forked(run_service, client.port):
+        client = GrackleClient('localhost', 8436)
+        with fake_grackle_service(client):
             client.put_message('arch1', 'asdf', StringIO('This is a message'))
             with ExpectedException(Exception, 'wtf'):
                 client.put_message('arch1', 'asdf',
                     StringIO('This is not a message'))
 
 
-class TestMessagePath(TestCase):
+class TestGetMessages(TestCase):
 
-    def test_message_path(self):
-        self.assertEqual('/my%20/%3C', message_path('my ', '<'))
+    def test_get_messages(self):
+        client = GrackleClient('localhost', 8435)
+        with fake_grackle_service(client,
+            {'baz':
+            [{'message-id': 'foo'},
+             {'message-id': 'bar'}]}):
+            response = client.get_messages('baz', message_ids=['a'])
+        self.assertEqual(['bar', 'foo'], sorted(m['message-id'] for m in
+            response))

